function output = restricted_3body(t, m1_over_m2, r_0, rdot_0, theta_0, thetadot_0, z_0, zdot_0)
% Integration of the differential equations for the restricted three-body problem 
% analyzed in Section 4.6 (Eqs. 4.6.13 to 4.6.15).  All distances are normalized 
% by L (which is the distance between bodies P1 and P2)
% Example of usage ( using the initial conditions  
%      r_0/L=1.001; rdot_0/L=0; theta_0=59*pi/180; thetadot_0=0; z_0/L=0.01; zdot_0=0 ): 
% my_output = restricted_3body([0:0.01:100], 80,  1.001, 0, 59*pi/180, 0, 0.01, 0);

initial_conditions=[r_0; rdot_0; theta_0; thetadot_0; z_0; zdot_0];
[t, y]=ode45(@rhs_of_diff_eqs, t, initial_conditions, [],  m1_over_m2); 
output=[t, y];  
% The variable output is a n by 7 matrix, where n is the number of points in the array t 
% 1st column: t, 2nd column: r/L, 3rd column: d(r/L)/dt; 4th column: theta, 
% 5th column: d(theta)/dt; 6th column: z/L, 7th column: d(z/L)/dt  
r_over_L=y(:,1);   theta=y(:,3);   z_over_L=y(:,5);

close all  % closing (i.e., deleting) all figure windows
% Plots:
figure('color', 'white');   plot(t, r_over_L);   grid        % plot of r/L versus time t 
xlabel('t   (seconds)', 'fontsize', 12),    ylabel('r/L', 'fontsize', 12) 
title('Restricted three-body problem','fontsize', 12)

figure('color', 'white');   plot(t, theta*180/pi);   grid     % plot of theta versus time t         
xlabel('t   (seconds)', 'fontsize', 12),    ylabel(['\theta', ' (degrees)'], 'fontsize', 12) 
title('Restricted three-body problem','fontsize', 12)

figure('color', 'white');   plot(t, z_over_L);   grid  % plot of z/L versus t        
xlabel('t   (seconds)', 'fontsize', 12), ylabel('z/L', 'fontsize', 12) 
title('Restricted three-body problem','fontsize', 12)

x=r_over_L.*cos(theta); y=r_over_L.*sin(theta);    animate_nbars(x, y)  % animation
%  Plot of y/L versus x/L:
figure('color', 'white');   plot(r_over_L.*cos(theta), r_over_L.*sin(theta));  grid           
xlabel('x/L', 'fontsize', 12),   ylabel('y/L', 'fontsize', 12)  
title('Restricted three-body problem', 'fontsize', 12)


function dy_dt=rhs_of_diff_eqs(t, y,  m1_over_m2) 
    % Note:  y(1)=r/L, y(2)=d(r/L)/dt, y(3)=theta, y(4)=d(theta)/dt, y(5)=z/L, y(6)=d(z/L)/dt 
    cos_theta=cos(y(3));  sin_theta=sin(y(3));
    r31_cube= (y(1)^2+y(5)^2)^(3/2);  r32_cube= ( 1+y(1)^2+y(5)^2 -2*y(1)*cos_theta )^(3/2);
    dy_dt(1)= y(2);    % this is d(r/L)/dt     
    dy_dt(2)= y(1)*(1+y(4))^2+((cos(y(3))-y(1))/r32_cube-cos(y(3)) -m1_over_m2*y(1)/r31_cube)/(1+m1_over_m2); % this is d2(r/L)/dt2 from Eq. (4.6.13)
    dy_dt(3)= y(4);
    dy_dt(4)= ((1-1/r32_cube)*sin_theta/(1+m1_over_m2)-2*y(2)*(1+y(4)))/y(1); 
    dy_dt(5)= y(6);
    dy_dt(6)= -(m1_over_m2/r31_cube+1/r32_cube)*y(5)/(1+m1_over_m2);
    dy_dt=[dy_dt(1); dy_dt(2); dy_dt(3); dy_dt(4); dy_dt(5); dy_dt(6)];

% end of the file restricted_3body.m
