function output = two_body_simul(t, Re, g, m2_over_m1_plus_1, r_0, v_0, flight_path_angle_0)
% Animation of the two-body problem analyzed in Sections 4.4 and 4.5 of Chapter 4
% NOTE:   Re: in km, g: in m/s^2, r_0: in km, v_0: in km/hour, flight_path_angle_0: in degrees 
% Example of usage ( with the initial conditions  r_0=6578; v_0=28040; flight_path_angle_0=0 ):
% my_output = two_body_simul([0:0.001:30], 6378, 9.81, 1,   6578, 28040, 0);
% also try:   my_output = two_body_simul([0:0.001:30], 6378, 9.81, 1,   6578, 38040, 0);
%      and:   my_output = two_body_simul([0:0.001:30], 6378, 9.81, 1,   6578, 27040, 0);

disp(' ')
disp('NOTE: IN ADDITION TO ANIMATING THE TWO-BODY PROBLEM, A CALL SUCH AS ')
disp(' output=two_body_simul(t, Re, g, m2_over_m1_plus_1, r_0, v_0, flight_path_angle_0)') 
disp('RETURNS, IN THE VARIABLE output, 4 COLUMNS CONTAINING  time, r, dr/dt, and theta')
% *************************************************************************
disp(' '), disp('  =====>  IMPORTANT MESSAGE:  <=====')
disp(' IF THE SIMULATION RUNS EITHER TOO FAST OR TOO SLOW IN YOUR COMPUTER,')
disp(' change the time_step in the time array t=[t0:time_step:tfinal] of your simulation ')
disp(' ---> EITHER to a smaller value to slow down the animation ')
disp(' ---> OR to a larger value to speed up the animation')
% *************************************************************************
disp(' ')

dr_dt_0=v_0*sin(flight_path_angle_0*pi/180); theta0=0;
[t, y]=ode45(@rhs_diff_eqs, t, [r_0; dr_dt_0; theta0], [], Re, g, m2_over_m1_plus_1, r_0, v_0, flight_path_angle_0); 
output=[t, y];    r=output(:,2); theta=output(:,4);

% *************  Let's see if the satellite crashed  *****************************************
number_of_time_points=length(t);   newlen=number_of_time_points;  temp=find(r <= Re);
if ~isempty(temp) 
    temp1=temp(1)-1;  newlen=temp(1)-1; 
    tnew=t(1:newlen);  number_of_time_points=newlen;  
    disp(' '), disp(['THE SATELLITE CRASHES AFTER  t= ', num2str(tnew(newlen)),  ' hours'])
end
% Resize all the proper lists and then proceed with the new reduced time list tnew
tnew=t(1:newlen); t=tnew; r=r(1:newlen,1); theta=theta(1:newlen,1); output=[tnew, r, theta];
% ********************************************************************************************

temp1=r.*cos(theta); temp2=r.*sin(theta); 
x_min=min(temp1); x_max=max(temp1);  y_min=min(temp2); y_max=max(temp2);  
if x_min==x_max, disp('THE STEP SIZE IS TOO BIG; DECREASE IT TO RUN THE SIMULATION'),  return, end
 
close all  % deleting all existing figures
% Create a figure window for the animation. Start by determining the computer monitor size
% so that the animation window is properly placed and sized in any computer screen.  
screen_width_and_height=get(0,'screensize');     %  width and height of the screen, in pixels
PC_screen_width =screen_width_and_height(3);     % this is the width of the computer monitor
PC_screen_height=screen_width_and_height(4);     % this is the height of the computer monitor

x_topleft= 0.65*PC_screen_width;   % choice for the top left x-coordinate of the figure window
y_topleft= 0.17*PC_screen_height;  % choice for the top left y-coordinate of the figure window
top_left_coordinates=[x_topleft, y_topleft]; 
size_of_fig_window = [0.32*PC_screen_width, 0.47*PC_screen_height];  
% the above are the width and height chosen for the animation window
% Now, create the figure window.  
animation_fig=figure( 'position', [x_topleft, y_topleft,size_of_fig_window(1), size_of_fig_window(2)], 'name','Animation window', 'color', 'white', 'Doublebuffer', 'on');
% NOTE: if 'DoubleBuffer' is 'off' (the default), this animation will flicker, which is not desirable
  
xlabel('x (in km) from the planet center', 'fontsize', 12)  
ylabel('y (in km) from the planet center', 'fontsize', 12)  
title('The two-body problem', 'fontsize', 12)

h_axes=gca; set(h_axes, 'xlim', [-abs(x_min), x_max], 'ylim', [-abs(y_min), y_max], 'dataaspectratio', [1, 1, 0.1], 'NextPlot','add') 

th=0:0.02:2*pi;  plot(Re*cos(th), Re*sin(th));   % a circle representing the attracting planet
satellite= plot(1.05*Re, 0, 'o');  
rot_planet=plot([0,0], [0, 0]);    
% rot_planet will be a line that will represent the spinning central attracting planet

% record the Earth-day number in the figure window 
om=2*pi/24;  tau=om*t;  day_number=ceil(tau/(2*pi));  dia=num2str(day_number);
uicontrol(animation_fig,'style','text', 'position',[10,10,40,15], 'string', 'day');   
daynumber=uicontrol(animation_fig,'style','text', 'position',[40,10,40,15], 'string', ' '); 

for itime=1:number_of_time_points 
    set(satellite, 'xdata', r(itime,1)*cos(theta(itime,1)), 'ydata', r(itime,1)*sin(theta(itime,1) ) );   
    set(rot_planet, 'xdata', [0, Re*cos(om*t(itime,1))], 'ydata', [0, Re*sin(om*t(itime,1))] ); 
    drawnow; 
    set(daynumber, 'string', dia(itime) )  
end

function dy_dt=rhs_diff_eqs(t, y,  Re, g, m2_over_m1_plus_1, r_0, v_0, flight_path_angle_0) 
% State variabes: y(1)=r, y(2)=dr/dt, y(3)=theta;  
% unit of distance: km, unit of time: hour;  g: in m/s^2
       r_sq=y(1)^2;
       dtheta_dt = r_0*v_0*cos(flight_path_angle_0*pi/180)/r_sq;
       d2r_dt2 = y(1)*dtheta_dt^2 - m2_over_m1_plus_1*g*3600*3.6*(Re^2)/r_sq;
   dy_dt(1) = y(2);         % this is dr/dt 
   dy_dt(2) = d2r_dt2;      % this is d2(r)/dt2
   dy_dt(3) = dtheta_dt;    % this is d(theta)/dt
   dy_dt=[dy_dt(1); dy_dt(2); dy_dt(3)];
   
% end of the file two_body_simul.m


