function output = fourbar_kinematics(t, L1, L2, L3, L4, theta2, dtheta2_dt, d2theta2_dt2,  solution_number)
// Kinematics of the four-bar mechanism in Section 3.2.3
// Animation is done in the program fourbar_mechanism.sci 
// Example of usage [copy the next three lines, without the //, to the Scilab console]:
// t=[0:0.1:5]; theta2=%pi/6+1.5*%pi*sin(2*t);  dtheta2_dt=3*%pi*cos(2*t); d2theta2_dt2=-6*%pi*sin(2*t);
// my_output=fourbar_kinematics(t, 1, 0.45, 0.8, 0.7, theta2,  dtheta2_dt, d2theta2_dt2, 1);
//   NOTE: any name can be used for the variable to the left of the equal sign in line 1
//  The variable output is a n by 10 column, where n is the number of points in the array t
// 
// The mechanism is defined by four points A, B, C, and D, whose x-y coordinates are:
// point A: (0,0),   point B: ( L2*cos(theta2), L2*sin(theta2) )
// point C( L1+L4*cos(theta4), L4*sin(theta4) ),   point D: (L1, 0 )
// The angles theta3 and theta4 are theta3=output(2,:) and theta4=output(3,:)
// The fourth bar is a fictitious one between the fixed points A and D. 

len=length(t);  temp=ones(1,len); // This assures that theta2, dtheta2_dt, and d2theta2_dt2 
//      are rows with the same length as the row t
theta2=theta2.*temp; dtheta2_dt=dtheta2_dt.*temp; d2theta2_dt2=d2theta2_dt2.*temp;

if solution_number ==1 | solution_number == 2 then 
          cos_theta2=cos(theta2);  L=sqrt( L1^2+L2^2-2*L1*L2*cos_theta2 );   // L from Eq. (3.2.10)
          sin_theta2=sin(theta2);  sin_alpha=L2*sin_theta2./L;  // need to use ./, instead of / 
          if abs(sin_alpha) <=1 then
              cos_alpha = (L1-L2*cos(theta2))./L;        // from the equations of (3.2.9)
              angle_alpha=atan(sin_alpha, cos_alpha);    // angle angle_alpha
              cos_beta=(L3^2+L.^2-L4^2)./(2*L*L3);   
              if abs(cos_beta)<=1 then              
                  sin_beta=(3-2*solution_number)*sqrt(1-cos_beta.^2);  // Eqs. (3.2.14) and (3.2.15)
                  angle_beta= atan(sin_beta, cos_beta); 
                  theta3=angle_beta-angle_alpha;           // angle theta3 from Eq. (3.2.16) 
                  sin_gamma=L3*sin_beta/L4;                //  sin(gamma) from Eq. (3.12.11)
                  if abs(sin_gamma)<=1 then    
                      cos_gamma=(L-L3*cos_beta)/L4;        //  cos(gamma) from Eq. (3.12.12)
                      angle_gamma= atan(sin_gamma, cos_gamma);      
                      theta4=%pi-angle_alpha-angle_gamma;
                  else disp('THE MECHANISM CANNOT OPERATE because |sin(gamma)|>1'), output=[], return
                  end
              else disp('THE MECHANISM CANNOT OPERATE because |cos(beta)|>1'),      output=[], return
              end
          else disp('THE MECHANISM CANNOT OPERATE because |sin(alpha))|>1'),        output=[], return
          end

// Angular velocities dtheta3_dt=d(theta3)/dt and dtheta4_dt=d(theta4)/dt from Eq. (3.2.5):
den=L3*L4*sin(theta4-theta3);  // this is the determinant of the 2 by 2 matrix in Eq. (3.2.5)
temp=-L2*dtheta2_dt./den;
// Solution of Eq. (3.2.5):
dtheta3_dt=L4*temp.*sin(theta4-theta2);  dtheta4_dt=L3*temp.*sin(theta3-theta2);     

// Elements a11, a12, a21, and a22 of the inverse of the matrix in Eq. (3.2.5):
cos_theta3=cos(theta3); sin_theta3=sin(theta3);  
cos_theta4=cos(theta4); sin_theta4=sin(theta4);
a11=-L4*cos_theta4./den; a12=L4*sin_theta4./den; 
a21=-L3*cos_theta3./den;  a22=L3*sin_theta3./den;  

// Angular accelerations d2theta3_dt2 and d2theta4_dt2 from Eq. (3.2.8):
v1=  L4*(dtheta4_dt.^2).*cos_theta4 -L3*(dtheta3_dt.^2).*cos_theta3 - L2*( d2theta2_dt2.*sin_theta2 + (dtheta2_dt.^2).*cos_theta2 );
v2= -L4*(dtheta4_dt.^2).*sin_theta4 +L3*(dtheta3_dt.^2).*sin_theta3 - L2*( d2theta2_dt2.*cos_theta2 - (dtheta2_dt.^2).*sin_theta2 );
d2theta3_dt2=a11.*v1+a12.*v2; d2theta4_dt2=a21.*v1+a22.*v2;  
 // d2theta3_dt2 and d2theta4_dt2 are the solutions of Eq. (3.2.8) 

// Store t, all the angles, angular velocities, and angular accelerations in the variable 
// output so that they are available to the user:
temp1=[t; theta2;theta3;theta4; dtheta2_dt;dtheta3_dt;dtheta4_dt]';  // this is to avoid a long line
temp2=[d2theta2_dt2;d2theta3_dt2;d2theta4_dt2]';                     // and so is this
output=[temp1, temp2];

  else disp(' -----> SOLUTION_NUMBER HAS TO BE  1 or 2 <----- '),    output=[], return
end
endfunction      // end of the file fourbar_kinematics.sci
