function output=fourbar_mech_dynamics(t, L1, L2, L3, L4, m2, m3, m4, theta2_0, dtheta2_dt_0, solution_number) 
// Simulation for Example 7.9.3 in Section 7.9 of Chapter 7
// Strings used for names of variables are limited in Scilab to 24 characters.  
// That is why a name like fourbar_mechanism_dynamics has to be shortened
// Example of usage [copy the line below to the Scilab console, without the //]: 
// my_output=fourbar_mech_dynamics([0:0.002:5], 1, 0.4, 0.8, 0.7, 3.75, 3, 3,  %pi/6, 0, 1); 
//
// NOTE: for these values of L1, L3, and L4, the mechanism locks shortly after t=1.2 if 
// L2 > approximately 0.5.  If the mechanism locks, a standard error message will appear  
// on the computer screen, and the computation ends. If this occurs, ignore the message  
// but redesign the mechanism by changing the length of at least one of its bars to 
// prevent it from locking. 
//
// IN ADDITION TO ANIMATING THE MECHANISM, SUCH A CALL TO THE FUNCTION 
// fourbar_mech_dynamics.sci GENERATES A len (WHICH IS THE NUMBER OF POINTS 
// IN THE SPECIFIED ARRAY t) BY 10 MATRIX CALLED output (OR ANY OTHER NAME 
// YOU WANT TO USE BEFORE THE EQUAL SIGN SEEN IN LINE 1), WHOSE COLUMNS 
// ARE t, theta2, theta3, theta4, d(theta2)/dt, d(theta3)/dt, d(theta4)/dt, d2(theta2)/dt2, 
//        d2(theta3)/dt2, d2(theta4)/dt2 
//
//  my_output(:,4), my_output(:,2)*180/%pi, FOR EXAMPLE, GIVE theta4 (IN RAD) AND theta2 
// (IN DEGREES). TO PLOT theta4 IN DEG VERSUS TIME t, FOR EXAMPLE, TYPE, 
//  IN THE SCILAB CONSOLE, THE COMMANDS 
//   figure(); plot(my_output(:,1), my_output(:,4)*180/(2*%pi) )
//   To put a grid on the plot, if desired, type the command   xgrid
//
// In this problem theta2 and d(theta2)/dt are not given; only their initial conditions 
// theta2_0 and dtheta2_dt_0. The system is driven by a motor with a torque M_motor that is 
// a specified function of time in the function M_motor(t), which can be changed by the user.  
//
// The variables theta2 and d(theta2)/dt are determined by integrating a second-order 
// differential equation obtained from the firt row of the solution of Eq. (7.9.3n). The 
// integration is done by a call to the Scilab function ode [which calls the function 
// rhs_diff_eqs(t, y) that specifies the right hand side of the equations 
// dtheta2/dt= --- , and d(dtheta2/dt)/d t= ---].  

if solution_number ~=1 & solution_number ~= 2 then 
    disp('--> SOLUTION_NUMBER HAS TO BE 1 or 2 <---'), output=[], return, end 

 disp(' '),  disp('The function fourbar_mech_dynamics called fourbar_mechanism.')
   
function M_motor_of_t= M_motor(t)
   M_motor_of_t=%pi/6+1.5*%pi*sin(2*t) // NOTE: change this expression, if desired
endfunction   

function dy_dt = rhs_diff_eqs(t, y) // this is called by ode to integrate the diff. eq.  
    //state variables: y(1)=theta2,  y(2)=dtheta2_dt
 theta2=y(1),  dtheta2_dt=y(2)
 d2theta2_dt2 = ang_acceleration(t, L1, L2, L3, L4,  m2, m3, m4, theta2, dtheta2_dt, solution_number)
 dy_dt=[y(2), d2theta2_dt2] 
endfunction         // end of the rhs_diff_eqs function

function d2theta2_dt2 = ang_acceleration(t, L1, L2, L3, L4,  m2, m3, m4, theta2, dtheta2_dt, solution_number)
fourbar = fourbar_kinematics(t, L1, L2, L3, L4, theta2, dtheta2_dt, d2theta2_dt2, solution_number)
// Here we have valid values for theta2, theta3, theta4, d(theta2)/dt, d(theta3)/dt, and d(theta4)/dt 
// Ignore d2theta3_dt2 and d2theta4_dt2 obtained by fourbar_kinematics

 theta2=fourbar(:,2);  theta3=fourbar(:,3);  theta4=fourbar(:,4);   
 sin_theta2=sin(theta2);  sin_theta3=sin(theta3);  sin_theta4=sin(theta4);
 cos_theta2=cos(theta2);  cos_theta3=cos(theta3);  cos_theta4=cos(theta4);
 sin43=sin(theta4-theta3);  cos43=cos(theta4-theta3);      
 dtheta2_dt=fourbar(:,5); dtheta3_dt=fourbar(:,6); dtheta4_dt=fourbar(:,7); 
 dtheta2_dt_sq=dtheta2_dt^2, dtheta3_dt_sq=dtheta3_dt^2, dtheta4_dt_sq=dtheta4_dt^2
 
// angular acceleration d2theta2_dt2=d2(theta2)/dt2 from Eq. (7.9.3n)
 M=[m2*L2^2/3,       0,                   0,                 L2*sin_theta2,  -L2*cos_theta2;
    0,               m3*L3^2/3,          -m3*L3*L4*cos43/2,  L3*sin_theta3,  -L3*cos_theta3;
    0,              -m3*L3*L4*cos43/2,   (m4/3+m3)*L4^2,    -L4*sin_theta4,   L4*cos_theta4;
    L2*sin_theta2,   L3*sin_theta3,      -L4*sin_theta4,     0,               0;
    L2*cos_theta2,   L3*cos_theta3,      -L4*cos_theta4,     0,               0]; 
 Minv=inv(M);     // M is the matrix seen in Eq. (7.9.3n)), and Minv is its inverse

if (abs(rcond(M))) <= 1.e-7 then
          disp('THE MECHANISM IS ABOUT TO LOCK. SUGGESTION: STOP THE SIMULATION.')
output=[ ], return, end

 v1= M_motor(t);   v2=-m3*L3*L4*dtheta4_dt_sq*sin43/2;  v3=m3*L3*L4*dtheta3_dt_sq*sin43/2;  
 v4=  L4*dtheta4_dt_sq*cos_theta4 -L3*dtheta3_dt_sq*cos_theta3 -L2*dtheta2_dt_sq*cos_theta2;
 v5= -L4*dtheta4_dt_sq*sin_theta4 +L3*dtheta3_dt_sq*sin_theta3 +L2*dtheta2_dt_sq*sin_theta2;   
 d2theta2_dt2=Minv(1,1)*v1 +Minv(1,2)*v2 +Minv(1,3)*v3 +Minv(1,4)*v4 +Minv(1,5)*v5;
endfunction        // end of the ang_acceleration function

  // Integration of the second-order diff. eq. for theta2. An arbitrary value is used for
 // d2theta2_dt2 because one is needed in a call to fourbar_mechanism
 d2theta2_dt2=0; // this is an arbitrary value  
 state_variables = ode([theta2_0; dtheta2_dt_0], 0, t, rhs_diff_eqs) 
 theta2=state_variables(1,:);   dtheta2_dt=state_variables(2,:);     // this is the solution 

 // We now determine the column of valid values for d2theta2/dt2. After that is done,  
 // call fourbar_mechanism to complete the solution and animate the motion
  len=length(theta2); temp=[]; 
  for k=1:len, 
   temp=[temp, ang_acceleration(t(k), L1, L2, L3, L4,  m2, m3, m4, theta2(k), dtheta2_dt(k), solution_number)]
  end 
  d2theta2_dt2=temp;
  output = fourbar_mechanism(t, L1, L2, L3, L4, theta2, dtheta2_dt, d2theta2_dt2, solution_number)
  
// disp(' ')
//  disp('MESSAGE FROM THE FUNCTION fourbar_mech_dynamics.sci:') 
// disp('THE 10 COLUMN MATRIX GENERATED BY THE FUNCTION fourbar_mech_dynamics.sci')
// disp('CONTAINS:  t, theta2, theta3, theta4, dtheta2_dt, dtheta3_dt, ')
// disp('   dtheta4_dt,  d2theta2_dt2, d2theta3_dt2, d2theta4_dt2 ')             
endfunction    // end of file fourbar_mech_dynamics

