function  output = fourbarforces(t, L1, L2, L3, L4, theta2, dtheta2_dt, d2theta2_dt2, solution_number, m2, m3, m4)
// Simulation of the problem in Section 5.6 of Chapter 5
//  Example of usage [copy the next three lines to the Scilab console, without the //]:
// t=[0:0.1:5]; theta2=%pi/6+1.5*%pi*sin(2*t);  dtheta2_dt=3*%pi*cos(2*t); d2theta2_dt2=-6*%pi*sin(2*t); 
// output=fourbarforces(t, 1, 0.45, 0.8, 0.7, theta2, dtheta2_dt, d2theta2_dt2, 1, 0.15, 0.225, 0.1875);
//
// THE FOLLOWING CALL GIVES THE SAME RESULTS SHOWN IN SECTION 5.6 OF CHAPTER 5:
// output=fourbarforces(1.e-6, 5.1592, 2, 3, 2.5, %pi/2-0.5902, -3*%pi, 0,  1,   0.15, 0.225, 0.1875);
//
// SUCH CALLS TO THE FUNCTION fourbarforces.sci GENERATE A len (WHICH IS THE NUMBER OF  
// POINTS IN THE SPECIFIED ARRAY t) BY 10 MATRIX CALLED output, WHOSE COLUMNS ARE
//      t, Mext, Ar2, An2, Dr4, Br2, Bn2, Cr4, Dn4, Cn4
//
//  output(:,3), AND output(:,4), FOR EXAMPLE, GIVE THE REACTIONS Ar2 AND An2 
//  TO PLOT Ar2 VERSUS TIME t, FOR EXAMPLE, TYPE, IN THE SCILAB CONSOLE, THE COMMANDS
//    figure();   plot( output(:,1), output(:,3) )

disp(' ')
disp(' NOTE: THE 10 VARIABLES t, Mext, Ar2, An2, Dr4, Br2, Bn2, Cr4, Dn4, and Cn4, AT THE')
disp('SPECIFIED TIMES, ARE STORED IN A 10-COLUMN MATRIX WHOSE NAME IS THE ONE YOU USED IN')
disp('THE CALL TO THE FUNCTION fourbarforces.sci, SUCH AS   output=fourbarforces(....)' )
disp(' TO PLOT Ar2, for example, VERSUS time, USE THE COMMANDS ');
disp('      figure(); plot(output(:,1), output(:,3))')
disp('TO PUT A GRID ON THE PLOT, IF DESIRED, THEN TYPE  xgrid')
disp(' --> To see an animation of the motion, use the function fourbar_mechanism <---') 
disp(' ')

// First call four_bar_kinematics to get    theta2, theta3, theta4, dtheta2_dt, dtheta3_dt,  
//                                       dtheta4_dt,, d2theta2_dt2, d2theta3_dt2, d2theta4_dt2
fourbar=fourbar_kinematics(t, L1, L2, L3, L4, theta2, dtheta2_dt, d2theta2_dt2, solution_number)

if fourbar==[] then output=[], return, end

theta2=fourbar(:,2); theta3=fourbar(:,3); theta4=fourbar(:,4); 
dtheta2_dt=fourbar(:,5); dtheta3_dt=fourbar(:,6); dtheta4_dt=fourbar(:,7);  
d2theta2_dt2=fourbar(:,8); d2theta3_dt2=fourbar(:,9); d2theta4_dt2=fourbar(:,10); 
// The variables theta2, dtheta2_dt, ...,  d2theta4_dt2 are columns

// We can now calculate the support reactions at each time instant specified in the array t
Cn4=m4*L4*d2theta4_dt2/3;  Dn4=Cn4/2   // From Eqs. (5.6.10) and (5.6.11)

// Now, determine the reactions Br2, Bn2, and Cr4 from Eq. (5.6.12). We will use the 
// analytical expression for the inverse of the matrix that appears in Eq. (5.6.12), 
// which is obtained after some algebra that can be done by hand. Its elements are 
// a11, a12, a13, etc, a33, shown below. 

cos32=cos(theta3-theta2);     sin32=sin(theta3-theta2);    
cos42=cos(theta4-theta2);     sin42=sin(theta4-theta2);
cos43=cos(theta4-theta3);     sin43=sin(theta4-theta3);
cos432=cos(theta4+theta3-2*theta2); sin432=sin(theta4+theta3-2*theta2);

den=4*sin43;  // this is the determinant of the matrix in Eq. (5.6.12). 
// The elements of the inverse of that matrix are:
a11=(sin432+3*sin43)./den;   a12=-(cos43+cos432)./den;    a13=2*cos42./den;
a21=(cos43-cos432)./den;     a22=(3*sin43-sin432)./den;   a23=2*sin42./den;           
a31=-2*sin32./den;           a32=2*cos32./den;            a33=-2*ones(length(t),1)./den; 

dtheta2_dt_sq = (dtheta2_dt.^2); dtheta3_dt_sq = (dtheta3_dt.^2); dtheta4_dt_sq = (dtheta4_dt.^2);
// Elements of the column array seen in Eq. (5.6.12):
v1= m3*L3*(  dtheta3_dt_sq.*cos32 + d2theta3_dt2.*sin32 )/2 + m3*L2*dtheta2_dt_sq + Cn4.*sin42;
v2=-m3*L3*( -dtheta3_dt_sq.*sin32 + d2theta3_dt2.*cos32 )/2 - m3*L2*d2theta2_dt2  - Cn4.*cos42;
v3= m3*L3*d2theta3_dt2/6 + Cn4.*cos43;

// We then have for the reactions Br2, Bn2, and Cr4, from Eq. (5.6.12):
Br2=a11.*v1+a12.*v2+a13.*v3;  Bn2= a21.*v1+a22.*v2+a23.*v3;  Cr4= a31.*v1+a32.*v2+a33.*v3;

// The remaining reactions are:
Mext= m2*L2^2*d2theta2_dt2/3 - L2*Bn2;      // From Eq. (5.6.3) 
Ar2= -m2*L2*dtheta2_dt_sq/2-Br2;            // From Eq. (5.6.1)   
An2= m2*L2*d2theta2_dt2/2-Bn2;              // From Eq. (5.6.2)    
Dr4= -m4*L4*dtheta4_dt_sq/2-Cr4;            // From Eq. (5.6.4)   

// disp(' ')
len=length(t);  
mprintf('At t= %5.3f, the reactions are: ', t($))   // t($) gives the value of the last element of the array t
disp(' ')
mprintf('Mext=%5.3f, Ar2=%f, An2=%f, Dr4=%f', Mext(len), Ar2(len), An2(len), Dr4(len) )
disp(' ')
mprintf('Br2=%f, Bn2=%f, Cr4=%f, Dn4=%f, Cn4=%f', Br2(len), Bn2(len), Cr4(len), Dn4(len), Cn4(len))
disp(' ')

  output=[t', Mext, Ar2, An2, Dr4, Br2, Bn2, Cr4, Dn4, Cn4];
endfunction      // end of the file  fourbaforces.sci   
