function output = restricted_3body(t, m1_over_m2, r_0, rdot_0, theta_0, thetadot_0, z_0, zdot_0)
// Integration of the differential equations for the restricted three-body problem  
// analyzed in Section 4.6 (Eqs. 4.6.13 to 4.6.15). All distances are normalized by L 
// (which is the distance between bodies P1 and P2)
// Example of usage ( using the initial conditions  
// r(0)/L=1.001; rdot(0)/L=0; theta(0)=59*%pi/180;  thetadot(0)=0; z_0/L=0.01; zdot_0=0 ): 
// my_output = restricted_3body([0:0.02:100], 80,  1.001, 0, 59*%pi/180, 0, 0.01, 0);

function dy_dt=rhs_of_diff_eqs(t, y) 
// Note:  y(1)=r/L, y(2)=d(r/L)/dt, y(3)=theta, y(4)=d(theta)/dt, y(5)=z/L, y(6)=d(z/L)/dt 
    cos_theta=cos(y(3)), sin_theta=sin(y(3))
    r31_cube= (y(1)^2+y(5)^2)^(3/2),  r32_cube= ( 1+y(1)^2+y(5)^2 -2*y(1)*cos_theta )^(3/2)
    dy_dt(1)= y(2)    // this is d(r/L)/dt     
    dy_dt(2)= y(1)*(1+y(4))^2+((cos(y(3))-y(1))/r32_cube-cos(y(3)) -m1_over_m2*y(1)/r31_cube)/(1+m1_over_m2) // this is d2(r/L)/dt2 from Eq. (4.6.13)
    dy_dt(3)= y(4)
    dy_dt(4)= ((1-1/r32_cube)*sin_theta/(1+m1_over_m2)-2*y(2)*(1+y(4)))/y(1) 
    dy_dt(5)= y(6)
    dy_dt(6)= -(m1_over_m2/r31_cube+1/r32_cube)*y(5)/(1+m1_over_m2)
endfunction

initial_conditions=[r_0; rdot_0; theta_0; thetadot_0; z_0; zdot_0]
state_variables=ode(initial_conditions, 0, t, rhs_of_diff_eqs)
output=[t', state_variables']  
// The variable output is a n by 7 matrix, where n is the number of points in the  array t
// column 1: t, column 2: r/L, column 3: d(r/L)/dt; column 4: theta, column 5: d(theta)/dt; 
// column 6: z/L, column 7: d(z/L)/dt   
r_over_L=output(:,2);  theta=output(:,4);  z_over_L=output(:,6)

xdel(winsid())  // deleting all existing figures
// Plots:
figure('background', 8);  plot(t', r_over_L),   xgrid      // plot of r/L versus time t 
xlabel('t   (seconds)', 'fontsize', 3), ylabel('r/L', 'fontsize', 3) 
title('Restricted three-body problem','fontsize', 3)

figure('background', 8);  plot(t', theta*180/%pi), xgrid  // plot of theta versus time t         
xlabel('t   (seconds)', 'fontsize', 3), ylabel(['$\theta$', ' (degrees)'], 'fontsize', 3) 
title('Restricted three-body problem' ,'fontsize', 3)

figure('background', 8);  plot(t', z_over_L),   xgrid     // plot of z/L versus t        
xlabel('t   (seconds)', 'fontsize', 3), ylabel('z/L', 'fontsize', 3)
title('Restricted three-body problem', 'fontsize', 3)

// plot of y/L versus x/L
figure('background', 8); plot(r_over_L.*cos(theta), r_over_L.*sin(theta)), xgrid         
xlabel('x/L', 'fontsize', 3), ylabel('y/L', 'fontsize', 3)
title('Restricted three-body problem', 'fontsize', 3)

  x=r_over_L.*cos(theta), y=r_over_L.*sin(theta),  animate_nbars(x, y)  //animation 
endfunction      // end of the file restricted_3body.sci



