function  output=slider_crank(t, D, L1, L2, theta1, dtheta1_dt, d2theta1_dt2, solution_number)
// Animation of the slider_crank mechanism in Section 3.4 using the function animate_nbars
// Examples of usage:
// t=[0:0.002:5]; theta1=2*%pi*t-%pi/6; dtheta1_dt=2*%pi; d2theta1_dt2=0; 
// my_output = slider_crank(t, -0.5, 1, 2.5, theta1, dtheta1_dt, d2theta1_dt2,   1);
// Example 2: 
// t=1/12; theta1=2*%pi*t-%pi/6; dtheta1_dt=2*%pi; d2theta1_dt2=0; 
// my_output = slider_crank(t, -0.5, 1, 2.5, theta1, dtheta1_dt, d2theta1_dt2,   1);
// Example 3: 
// t=[0:0.002:0.5]; theta1=2*%pi*t-%pi/6; dtheta1_dt=2*%pi; d2theta1_dt2=0; 
// my_output = slider_crank(t, -0.5, 1, 2.5, theta1, dtheta1_dt, d2theta1_dt2,   1);
//
// To slow down the animation, when desired, choose a smaller time step for a desired array t  
// Conversely, to speed up the animation, choose a larger time step 
//
// IN ADDITION TO ANIMATING THE MECHANISM, SUCH A CALL TO THE FUNCTION 
// slider_crank.sci GENERATES A len (WHERE len IS THE NUMBER OF 
// POINTS IN THE ARRAY t) BY 8 MATRIX CALLED my_output, WHOSE COLUMNS ARE
//   time, theta1, theta2, xB, dtheta2_dt, dxB_dt, d2theta2_dt2, d2xB_dt2
// my_output(:,3)*180/%pi, FOR EXAMPLE, GIVES theta2 (IN DEGREES). 
// TO PLOT IT VERSUS TIME t, TYPE, IN THE SCILAB CONSOLE, THE COMMANDS    
//     figure(); plot( my_output(:,1), my_output(:,3)*180/(2*%pi) )
// To put a grid on the plot, type   xgrid

disp('NOTE: IN ADDITION TO ANIMATING THE MECHANISM, A CALL SUCH AS')
disp('t=[0:0.02:5]; theta1=2*%pi*t-%pi/6; dtheta1_dt=2*%pi; d2theta1_dt2=0;')
disp('output=slider_crank(t, -0.5, 1, 2.5, theta1, dtheta1_dt, d2theta1_dt2, 1, 1);')
disp('RETURNS, IN THE VARIABLE output, 8 COLUMNS CONTAINING')
disp(' time, theta1, theta2, xB, dtheta2_dt, dxB_dt, d2theta2_dt2, d2xB_dt2') 
disp(' (the time array t only has those times for which the mechanism does not lock)') 

if solution_number ==1 | solution_number == 2 then 
    // First, make sure that theta1, dtheta1_dt, and d2theta1_dt2 have the same length as t
len=length(t); temp=ones(1,len);  newlen=len    
theta1=theta1.*temp; dtheta1_dt=dtheta1_dt.*temp; d2theta1_dt2=d2theta1_dt2.*temp;
 
// Make the simulation proceed only while |sin_theta2| <=1 
sin_theta2=(L1*sin(theta1)-D)/L2              // sin(theta2) from Eq. (3.4.6)
// Let's see if this is between -1 and 1:
temp=find( abs(sin_theta2)>1 ) 
if temp ~= [] then  
    temp1=temp(1)-1  
    if temp1 == 1 then
        output = [ ]  
        disp('The mechanism, as specified, does not work because the condition')
        disp('|L1*sin(theta1)-D|>=L2 is violated at the initial time') 
        return
    else
        newlen=temp1,   tnew=t(1:newlen)   // redefining the time array
        mprintf('The mechanism, as specified, stops after t= %5.3f', tnew(newlen) ) 
        mprintf(' because |L1*sin(theta1)-D|>L2 sometime thereafter.') 
    end
end

// Resize all the proper lists and do the computations.
tnew=t(1:newlen)
theta1=theta1(1:newlen); dtheta1_dt=dtheta1_dt(1:newlen); d2theta1_dt2=d2theta1_dt2(1:newlen)
sin_theta1 = sin(theta1);    cos_theta1 = cos(theta1)
sin_theta2=(L1*sin(theta1)-D)/L2    // sin(theta2) from Eq. (3.4.6), with the resized theta1
cos_theta2=(3-2*solution_number).*sqrt(1-sin_theta2.^2)  
// The above expression for cos(theta2) is from Eqs. (3.4.8) and (3.4.9)
xB=L1*cos_theta1+L2*cos_theta2    // xB from Eq. (3.4.7) 
theta2= atan(sin_theta2, cos_theta2)

xdel(winsid())  // deleting all figure windows
// Now, generate the cordinate matrices needed for the function animate_nbars: 
xO=zeros(newlen,1),  yO=xO,     xA=L1*cos_theta1',  yA=L1*sin_theta1'    // bar OA
x_coordinates=[xO, xA,  xB'],  y_coordinates=[yO, yA,  D*ones(newlen,1)] 
animate_nbars(x_coordinates, y_coordinates)   // animation

// Angulat velocity dtheta2_dt=d(theta2)/dt and velocity dxB_dt=d(xB)/dt:
dtheta2_dt=L1*dtheta1_dt.*cos_theta1./(L2*cos_theta2)      // d(theta2)/dt from Eq. (3.4.10)
dxB_dt=-L1*dtheta1_dt.*sin_theta1-L2*dtheta2_dt.*sin_theta2    // d(xB)/dt from Eq. (3.4.11)

// d2(theta2)/dt2 from Eq. (3.4.12), and d2(xB)/dt2 from Eq. (3.4.13):
d2theta2_dt2 = ( L1*(d2theta1_dt2.*cos_theta1-(dtheta1_dt.^2).*sin_theta1)/L2 + (dtheta2_dt.^2).*sin_theta2 )./cos_theta2
d2xB_dt2=-L1*(d2theta1_dt2.*sin_theta1+(dtheta1_dt.^2).*cos_theta1) - L2*(d2theta2_dt2.*sin_theta2+(dtheta2_dt.^2).*cos_theta2)

output=[tnew; theta1; theta2; xB; dtheta2_dt; dxB_dt; d2theta2_dt2; d2xB_dt2]'

else disp(' -----> SOLUTION_NUMBER HAS TO BE  1 or 2 <----- '),    output=[ ], return
end

endfunction      // end of the file  slider_crank.sci
